
"""Test cases for the q-gram index module"""

# pylint: disable=W0212

import unittest
from qgram import QGramIndex

class QGramChunkTest(unittest.TestCase):
    """Test the  implementation"""

    def test_long(self):
        """Test the chunk method with a long string"""
        index = QGramIndex()
        qgrams = index._chunk("weather")
        self.assertEqual(qgrams, ["$we", "wea", "eat", "ath", "the", "her", "er$"])

        index = QGramIndex(q_param=3)
        qgrams = index._chunk("hello")
        self.assertEqual(qgrams, ["$he", "hel", "ell", "llo", "lo$"])

    def test_short(self):
        """Test the chunk method with a short string"""
        index = QGramIndex()
        qgrams = index._chunk("eat")
        self.assertEqual(qgrams, ["$ea", "eat", "at$"])

        qgrams = index._chunk("to")
        self.assertEqual(qgrams, ["$to", "to$"])

        qgrams = index._chunk("a")
        self.assertEqual(qgrams, ["$a$"])

    def test_empty(self):
        """Test the chunk method with a empty string"""
        index = QGramIndex()
        qgrams = index._chunk("")
        self.assertEqual(qgrams, ["$$"])


    def test_q(self):
        """Test the chunk with alternative q's"""
        index = QGramIndex(q_param=5)
        qgrams = index._chunk("weather")
        self.assertEqual(qgrams, ["$weat", "weath", "eathe", "ather", "ther$"])

        qgrams = index._chunk("play")
        self.assertEqual(qgrams, ["$play", "play$"])

    def test_q_short(self):
        """Test the chunk with alternative q's and short strings"""
        index = QGramIndex(q_param=5)
        qgrams = index._chunk("eat")
        self.assertEqual(qgrams, ["$eat$"])

        qgrams = index._chunk("a")
        self.assertEqual(qgrams, ["$a$"])

    def test_q_empty(self):
        """Test the chunk with alternative q's and empty strings"""
        index = QGramIndex(q_param=5)
        qgrams = index._chunk("")
        self.assertEqual(qgrams, ["$$"])

class QGramNQGramsTest(unittest.TestCase):
    """Test the n_qgrams implementation"""

    def test_long(self):
        """Test the n_qgrams method with a long string"""
        index = QGramIndex()
        self.assertEqual(index._n_qgram("weather"), 7)

        index = QGramIndex(q_param=3)
        self.assertEqual(index._n_qgram("hello"), 5)

    def test_short(self):
        """Test the n_qgrams method with a short string"""
        index = QGramIndex()
        self.assertEqual(index._n_qgram("eat"), 3)
        self.assertEqual(index._n_qgram("to"), 2)
        self.assertEqual(index._n_qgram("a"), 1)

    def test_empty(self):
        """Test the n_qgrams method with a empty string"""
        index = QGramIndex()
        self.assertEqual(index._n_qgram(""), 1)


    def test_q(self):
        """Test the n_qgrams with alternative q's"""
        index = QGramIndex(q_param=5)
        self.assertEqual(index._n_qgram("weather"), 5)
        self.assertEqual(index._n_qgram("play"), 2)

    def test_q_short(self):
        """Test the n_qgrams with alternative q's and short strings"""
        index = QGramIndex(q_param=5)
        self.assertEqual(index._n_qgram("eat"), 1)
        self.assertEqual(index._n_qgram("a"), 1)

    def test_q_empty(self):
        """Test the n_qgrams with alternative q's and empty strings"""
        index = QGramIndex(q_param=5)
        self.assertEqual(index._n_qgram(""), 1)

class QGramAddTest(unittest.TestCase):
    """Test the add_term(s) implementations"""

    def test_add(self):
        """Check that adding a term populates the internal index"""
        index = QGramIndex()
        index.add_term("hello")

        self.assertEqual(index._index, {
            "$he": {"hello"},
            "hel": {"hello"},
            "ell": {"hello"},
            "llo": {"hello"},
            "lo$": {"hello"},
        })

    def test_add_multiple(self):
        """Check that adding multiple terms populates the internal index"""
        index = QGramIndex()
        index.add_term("rope")
        index.add_term("pope")

        self.assertEqual(index._index["ope"], {"pope", "rope"})
        self.assertEqual(index._index["pe$"], {"pope", "rope"})
        self.assertEqual(index._index["rop"], {"rope"})
        self.assertEqual(index._index["pop"], {"pope"})
        self.assertEqual(index._index["dop"], set())


class QGramSearchTest(unittest.TestCase):
    """Test the search implementations"""

    def test_find(self):
        """Check search() finds similar terms"""
        index = QGramIndex()
        index.add_term("hello")
        index.add_term("world")
        index.add_term("beer")
        index.add_term("moon")

        result = index.search("bear")

        self.assertEqual(result[0][1], "beer")
        self.assertEqual(len(result), 1)

    def test_no_find(self):
        """Check search() finds similar terms"""
        index = QGramIndex()
        index.add_term("hello")
        index.add_term("world")
        index.add_term("beer")
        index.add_term("moon")
        index.add_term("deer")

        result = index.search("baseball")

        self.assertEqual(len(result), 0)
